@@ -33,10 +33,10 @@ module LaunchpadHelper
 
   def ubuntu_targets_default
     [
-      ["bionic", "18.04"],
       ["focal", "20.04"],
-      ["impish", "21.10"],
       ["jammy", "22.04"],
+      ["kinetic", "22.10"],
+      ["lunar", "23.04"],
     ]
   end
 
@@ -45,7 +45,7 @@ module LaunchpadHelper
   end
 
   def dput_configuration_name
-    env_value("DPUT_CONFIGUARATION_NAME")
+    env_value("DPUT_CONFIGURATION_NAME")
   end
 
   def dput_incoming
