@@ -1,5 +1,5 @@
 # Copyright(C) 2014-2021  Sutou Kouhei <kou@clear-code.com>
-# Copyright(C) 2020-2021  Horimoto Yasuhiro <horimoto@clear-code.com>
+# Copyright(C) 2020-2023  Horimoto Yasuhiro <horimoto@clear-code.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -119,6 +119,10 @@ class PackagesGroongaOrgPackageTask < PackageTask
     task windows: ["windows:clean"]
   end
 
+  def docker_image(os, architecture)
+    "ghcr.io/#{github_repository}-package:#{super}"
+  end
+
   def use_built_package?
     false
   end
@@ -152,7 +156,7 @@ class PackagesGroongaOrgPackageTask < PackageTask
                                                   workflow_file_name,
                                                   branch: branch)
     workflow_runs_response.workflow_runs.each do |workflow_run|
-      artifacts_response = client.get(workflow_run.artifacts_url)
+      artifacts_response = client.get(workflow_run.artifacts_url, per_page: 50)
       next if artifacts_response.total_count.zero?
 
       artifacts_response.artifacts.each do |artifact|
@@ -206,6 +210,7 @@ class PackagesGroongaOrgPackageTask < PackageTask
     mkdir_p(repositories_dir)
     download_dir = "#{base_dir}/tmp/downloads/#{@version}"
     mkdir_p(download_dir)
+
     __send__("#{target_namespace}_targets").each do |target|
       branch = ENV["BRANCH"]
       if branch
@@ -253,9 +258,52 @@ class PackagesGroongaOrgPackageTask < PackageTask
         end
       end
     end
+
+    case target_namespace
+    when :source
+      rsync_dir =
+        "#{repository_rsync_base_path}/#{target_namespace}/#{@package}/"
+      sh("rsync",
+         "-av",
+         "--include=.htaccess",
+         "--exclude=*",
+         rsync_dir,
+         "#{repositories_dir}/")
+    end
   end
 
   def prepare(target_namespace)
+    case target_namespace
+    when :windows, :source
+      repositories_dir = download_repositories_dir(target_namespace)
+      mkdir_p(repositories_dir)
+
+      htaccess_path = "#{repositories_dir}/.htaccess"
+      if File.exist?(htaccess_path)
+        htaccess_content = File.read(htaccess_path)
+      else
+        htaccess_content = ""
+      end
+      File.open(htaccess_path, "w") do |htaccess|
+        htaccess_content.each_line do |line|
+          htaccess.puts(line) unless line.include?("-latest")
+        end
+        __send__("#{target_namespace}_targets").each do |target|
+          redirect_url = built_package_url(target_namespace, target)
+          [
+            target,
+            target.gsub(@version, "latest")
+          ].each do |redirect_target_file|
+            redirect_target =
+              "/#{target_namespace}/#{@package}/#{redirect_target_file}"
+            htaccess.puts("Redirect #{redirect_target} #{redirect_url}")
+            if target_namespace == :source
+              htaccess.puts("Redirect #{redirect_target}.asc #{redirect_url}.asc")
+            end
+          end
+        end
+      end
+    end
   end
 
   def release(target_namespace)
@@ -264,7 +312,19 @@ class PackagesGroongaOrgPackageTask < PackageTask
     destination = "#{repository_rsync_base_path}/"
     rsync_options = ["-av"]
     case target_namespace
-    when :source, :windows
+    when :source
+      sh({"GH_TOKEN" => github_access_token},
+         "gh",
+         "release",
+         "upload",
+         @version,
+         "--clobber",
+         "--repo", github_repository,
+         *Dir.glob("#{repositories_dir}/#{target_namespace}/#{@package}/*.asc"))
+      rsync_options << "--exclude=*.asc"
+      rsync_options << "--exclude=*.tar.gz"
+      rsync_options << "--exclude=*.zip"
+    when :windows
     else
       rsync_options << "--exclude=*.buildinfo"
       rsync_options << "--exclude=*.changes"
